import os
from typing import Dict, List

DIR = os.path.dirname(__file__)
FILES_PATH = "files/"

def get_abbreviation_map()->Dict[str,str]:
    file_name = "abbreviations.txt"
    unparsed = read_file(DIR, FILES_PATH, file_name)
    abbreviation_map = {j[0]: j[1] for j in [i.split(' : ') for i in unparsed]}
    return abbreviation_map

def get_abbreviation_list():
    abbreviation_list = list(get_abbreviation_map().keys())
    return abbreviation_list

def get_punctuation_list()->List[str]:
    file_name = "punctuation.txt"
    punctuation_list = read_file(DIR, FILES_PATH, file_name)
    return punctuation_list

def get_punctuation_string()->str:
    punctuation_list = get_punctuation_list()
    punctuation_string = ''.join(punctuation_list)
    return punctuation_string

def get_stopwords()->List[str]:
    file_name = "stopwords.txt"
    stopwords = read_file(DIR, FILES_PATH, file_name)
    return stopwords

def get_last_names()->List[str]:
    file_name = "last_names.txt"
    last_names = read_file(DIR, FILES_PATH, file_name)
    return last_names

def get_female_names()->List[str]:
    file_name = "ibge_female_names.txt"
    female_names = read_file(DIR, FILES_PATH, file_name)
    return female_names

def get_male_names()->List[str]:
    file_name = "ibge_male_names.txt"
    male_names = read_file(DIR, FILES_PATH, file_name)
    return male_names

def read_file(dir:str, relative_file_path:str, file_name:str)->List[str]:
    abs_file_path = os.path.join(dir, relative_file_path, file_name)
    with open(abs_file_path, "r") as txt_file:
        file_content = txt_file.read()
        content_list = file_content.split("\n")
        return content_list



from typing import List
from nltk import word_tokenize, sent_tokenize
from .cleaning import to_lower
from . import STOPWORDS, STEMMER_DICT

def tokenize(text:str, target:str='word')->List[str]:
    """
    tokenize can only target word or sentence
    """
    tokenize_function = word_tokenize if target == 'word' else sent_tokenize
    text_list =  tokenize_function(to_lower(text))
    return text_list

def detokenize(text_list:List[str])->str:
    new_text = " ".join(text_list)
    return new_text

def remove_stopwords(word_list:List[str], additional_word_list:List=[])->List[str]:
    stopwords = STOPWORDS   
    stopwords.extend(additional_word_list)
    new_word_list = [word for word in word_list if word not in stopwords]
    return new_word_list

def stem(word_list:List[str], stem_algo:str='rlsp')->List[str]:
    stemmer = STEMMER_DICT.get(stem_algo)
    new_word_list = [stemmer.stem(word) for word in word_list]
    return new_word_list



import re
from typing import List
import unicodedata
from . import PUNCTUATION_STRING, STOPWORDS

def remove_punctuation(text:str)->str:
    new_text = text.translate(str.maketrans('', '', PUNCTUATION_STRING))
    return new_text

def remove_accentuation(text:str)->str:
    new_text = ''.join(ch for ch in unicodedata.normalize('NFKD', text) if not unicodedata.combining(ch))
    return new_text

def remove_email(text:str)->str:
    new_text = re.sub(r'\S*@\S*\s?', ' ', text)
    return new_text

def remove_numbers(text:str)->str:
    new_text = re.sub(r"$\d+\W+|\b\d+\b|\W+\d+$", ' ', text)
    return new_text

def remove_html(text:str)->str:
    new_text = re.sub(r"<(?:\"[^\"]*\"['\"]*|'[^']*'['\"]*|[^'\">])+>",'',text)
    return new_text

def remove_url(text:str)->str:
    new_text = re.sub(r'(https?:\/\/(?:www\.)?[-a-zA-Z0-9@:%._+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}[-a-zA-Z0-9()@:%_+.~#?&/=]*)', ' ', text) 
    return new_text

def remove_extra_spaces(text:str)->str:
    new_text = " ".join(text.split()).strip()
    new_text = re.sub(r'\s+([?.!"])', r'\1', new_text)
    return new_text

def to_lower(text:str)->str:
    new_text = text.lower()
    return new_text



import os
import sys
import nltk
from nltk import PorterStemmer, LancasterStemmer, RSLPStemmer
file = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(os.path.realpath(file)))

from .utils import get_abbreviation_list, get_punctuation_list, get_punctuation_string, get_stopwords, get_abbreviation_map

nltk.download('punkt')

ABBREVIATION_MAP = get_abbreviation_map()
ABBREVIATION_LIST = get_abbreviation_list()
PUNCTUATION_LIST = get_punctuation_list()
PUNCTUATION_STRING = get_punctuation_string()

STOPWORDS = get_stopwords()

STEMMER_DICT = {
    'rlsp': RSLPStemmer(),
    'porter': PorterStemmer(),
    'lancaster': LancasterStemmer(),
}


# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open
from os import path

# The directory containing this file
HERE = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(HERE, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="pysquale",
    version="0.0.4",
    author="Henrique Santana",
    author_email="santana@cloudhumans.com",
    description="Useful lib for text processing in Portuguese",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://pysquale.readthedocs.io/",
    project_urls={
        "Bug Tracker": "https://github.com/santana-ai/pysquale/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(include=['pysquale', 'pysquale.*']),
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=[
        'nltk>=3.7',
    ],
)


import abc
from functools import reduce
from typing import Any, Callable, Iterable, List

class Pipeline(abc.ABC):
    def __init__(self, target:Any, items:Iterable[Any]):
        self.target = target
        self.items = items

    @abc.abstractmethod
    def run():
        pass

class ReducerPipeline(Pipeline):
    """
    It implements a Pipeline receiving just one item and passing it through multiple functions.
    The output of the previous function is the input of the following one.
    Ex: 
        text = "Olá, quem é você?"
        function_list = [to_lower, remove_accentuation, remove_punctuation]
    Pipeline: 
        "Olá, quem é você?" -> "olá, quem é você" -> "ola, quem e voce?" -> "ola quem e voce"
        output = "ola quem e voce"
    """
    def run(self)->Any:
        item:Any = self.target
        func_list:Iterable[Callable] = self.items
        output = reduce(
            lambda value, func: func(value),
            func_list,
            item,
        )
        return output

class MapperPipeline(Pipeline):
    """
    It implements a Pipeline receiving just one function and passing it through multiple items.
    The output of each function run is stored in a list of the same size of the input.
    Ex: 
        function = remove_accentuation
        text_list = ["Hortência", "Sebastião", "José"]
    Pipeline: 
        remove_accentuation("Hortência") -> remove_accentuation("Sebastião") -> remove_accentuation("José")
        output = ["Hortencia", "Sebastiao", "Jose"]
    """
    def run(self)->List[Any]:
        func:Callable = self.target
        items:Iterable = self.items
        output = list(map(lambda item: func(item), items))
        return output
